@@ -1808,6 +1808,10 @@ fm_list_view_start_renaming_file (FMDirectoryView *view, NautilusFile *file)
 		      NULL);
 
 	
+	gtk_tree_view_scroll_to_cell (list_view->details->tree_view,
+				      path,
+				      list_view->details->file_name_column,
+				      TRUE, 0.0, 1.0);
 	gtk_tree_view_set_cursor (list_view->details->tree_view,
 				  path,
 				  list_view->details->file_name_column,
