@@ -46,7 +46,6 @@
 #include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
-#include <eel/eel-generous-bin.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
@@ -90,6 +89,10 @@ struct _NautilusSpatialWindowDetails {
         guint save_geometry_timeout_id;	  
 	
 	GtkWidget *content_box;
+	GtkWidget *location_button;
+	GtkWidget *location_statusbar;
+
+	GnomeVFSURI *location;
 };
 
 GNOME_CLASS_BOILERPLATE (NautilusSpatialWindow, nautilus_spatial_window,
@@ -171,6 +174,28 @@ nautilus_spatial_window_unrealize (GtkWidget *widget)
 	}
 }
 
+
+static void
+nautilus_spatial_window_realize (GtkWidget *widget)
+{
+	NautilusSpatialWindow *window;
+
+	window = NAUTILUS_SPATIAL_WINDOW (widget);
+
+	GTK_WIDGET_CLASS (parent_class)->realize (widget);
+
+	if (window->loading) {
+		GdkCursor *cursor;
+
+		cursor = gdk_cursor_new (GDK_WATCH);
+		gdk_window_set_cursor (widget->window, cursor);
+		gdk_cursor_unref (cursor);
+	} else {
+		gdk_window_set_cursor (widget->window, NULL);
+	}
+}
+
+
 static void
 nautilus_spatial_window_destroy (GtkObject *object)
 {
@@ -190,6 +215,10 @@ nautilus_spatial_window_finalize (GObject *object)
 	
 	window = NAUTILUS_SPATIAL_WINDOW (object);
 
+	if (window->details->location != NULL) {
+		gnome_vfs_uri_unref (window->details->location);
+	}
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -283,6 +312,7 @@ static void
 real_merge_menus (NautilusWindow *nautilus_window)
 {
 	NautilusSpatialWindow *window;
+	BonoboControl *control;
 	BonoboUIVerb verbs [] = {
 		BONOBO_UI_VERB ("Close Parent Folders", file_menu_close_parent_windows_callback),
 		BONOBO_UI_VERB ("UpCloseCurrent", go_up_close_current_window_callback),
@@ -301,6 +331,12 @@ real_merge_menus (NautilusWindow *nautilus_window)
 
 	bonobo_ui_component_add_verb_list_with_data (nautilus_window->details->shell_ui,
 						     verbs, window);
+
+	control = bonobo_control_new (window->details->location_statusbar);
+	bonobo_ui_component_object_set (nautilus_window->details->shell_ui,
+		       			"/status/StatusButton",
+					BONOBO_OBJREF (control),
+					NULL);
 }
 
 static void
@@ -332,17 +368,203 @@ real_get_default_size(NautilusWindow *window, guint *default_width, guint *defau
    }
 }
 
+
+static void
+real_set_throbber_active (NautilusWindow *window, gboolean active)
+{
+	NautilusSpatialWindow *spatial;
+
+	spatial = NAUTILUS_SPATIAL_WINDOW (window);
+	spatial->loading = active;
+	
+	if (!GTK_WIDGET_REALIZED (GTK_WIDGET (window)))
+		return;
+
+	if (active) {
+		GdkCursor *cursor;
+
+		cursor = gdk_cursor_new (GDK_WATCH);
+		gdk_window_set_cursor (GTK_WIDGET (window)->window,
+				       cursor);
+		gdk_cursor_unref (cursor);
+		
+	} else {
+		gdk_window_set_cursor (GTK_WIDGET (window)->window,
+				       NULL);
+	}
+}
+
+
+static void
+location_menu_item_activated_callback (GtkWidget *menu_item,
+				       NautilusSpatialWindow *window)
+{
+	GnomeVFSURI *uri;
+	char *location;
+
+	uri = g_object_get_data (G_OBJECT (menu_item), "uri");
+	location = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+	nautilus_window_go_to (NAUTILUS_WINDOW (window), location);
+	g_free (location);
+	
+}
+
+static void
+menu_deactivate_callback (GtkWidget *menu,
+			  gpointer   data)
+{
+	GMainLoop *loop;
+
+	loop = data;
+
+	if (g_main_loop_is_running (loop)) {
+		g_main_loop_quit (loop);
+	}
+}
+
+static void
+menu_popup_pos (GtkMenu   *menu,
+		gint      *x,
+		gint      *y,
+		gboolean  *push_in,
+		gpointer	user_data)
+{
+	GtkWidget *widget;
+	GtkRequisition menu_requisition, button_requisition;
+
+	widget = user_data;
+
+	gtk_widget_size_request (GTK_WIDGET (menu), &menu_requisition);
+	gtk_widget_size_request (widget, &button_requisition);
+
+	gdk_window_get_origin (widget->window, x, y);
+
+	*y -= menu_requisition.height - button_requisition.height;
+
+	*push_in = TRUE;
+}
+
+static void
+location_button_clicked_callback (GtkWidget *widget, NautilusSpatialWindow *window)
+{
+	GtkWidget *popup, *menu_item;
+	GnomeVFSURI *uri;
+	char *name;
+	GMainLoop *loop;
+
+	g_return_if_fail (window->details->location != NULL);
+
+	popup = gtk_menu_new ();
+	
+	uri = gnome_vfs_uri_ref (window->details->location);
+	while (uri != NULL) {
+		name = nautilus_get_uri_shortname_for_display (uri);
+		menu_item = gtk_image_menu_item_new_with_label (name);
+		g_free (name);
+		gtk_widget_show (menu_item);
+		g_signal_connect (menu_item, "activate",
+				  G_CALLBACK (location_menu_item_activated_callback),
+				  window);
+		g_object_set_data_full (G_OBJECT (menu_item), "uri", uri, (GDestroyNotify)gnome_vfs_uri_unref);
+
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (popup), menu_item);
+
+		uri = gnome_vfs_uri_get_parent (uri);
+	}
+	gtk_menu_set_screen (GTK_MENU (popup), gtk_widget_get_screen (widget));
+
+	loop = g_main_loop_new (NULL, FALSE);
+
+	g_signal_connect (popup, "deactivate",
+			  G_CALLBACK (menu_deactivate_callback),
+			  loop);
+
+	gtk_grab_add (popup);
+	gtk_menu_popup (GTK_MENU (popup), NULL, NULL, menu_popup_pos, widget, 1, GDK_CURRENT_TIME);
+	g_main_loop_run (loop);
+	gtk_grab_remove (popup);
+	g_main_loop_unref (loop);
+ 	gtk_object_sink (GTK_OBJECT (popup));
+}
+
+void
+nautilus_spatial_window_set_location_button  (NautilusSpatialWindow *window,
+					      const char            *location)
+{
+	GnomeVFSURI *uri;
+	char *name;
+	
+	uri = NULL;
+	if (location != NULL) {
+		uri = gnome_vfs_uri_new (location);
+	}
+	if (uri != NULL) {
+		name = nautilus_get_uri_shortname_for_display (uri);
+		gtk_button_set_label (GTK_BUTTON (window->details->location_button),
+				      name);
+		g_free (name);
+		gtk_widget_set_sensitive (window->details->location_button, TRUE);
+	} else {
+		gtk_button_set_label (GTK_BUTTON (window->details->location_button),
+				      "");
+		gtk_widget_set_sensitive (window->details->location_button, FALSE);
+	}
+
+	if (window->details->location != NULL) {
+		gnome_vfs_uri_unref (window->details->location);
+	}
+	window->details->location = uri;
+}
+
 static void
 nautilus_spatial_window_instance_init (NautilusSpatialWindow *window)
 {
+	GtkShadowType shadow_type;
+	GtkWidget *frame;
+	GtkRcStyle *rc_style;
+
 	window->details = g_new0 (NautilusSpatialWindowDetails, 1);
 	window->affect_spatial_window_on_next_location_change = TRUE;
 
 	window->details->content_box = 
-		gtk_widget_new (EEL_TYPE_GENEROUS_BIN, NULL);
+		gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (window->details->content_box);
 	bonobo_window_set_contents (BONOBO_WINDOW (window), 
 				    window->details->content_box);
+
+	window->details->location_statusbar = gtk_statusbar_new ();
+	gtk_widget_show (window->details->location_statusbar);
+	gtk_widget_hide (GTK_STATUSBAR (window->details->location_statusbar)->frame);
+	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (window->details->location_statusbar), 
+					   FALSE);
+
+	window->details->location_button = gtk_button_new_with_label ("");
+	gtk_button_set_relief (GTK_BUTTON (window->details->location_button),
+			       GTK_RELIEF_NONE);
+	rc_style = gtk_widget_get_modifier_style (window->details->location_button);
+	rc_style->xthickness = 0;
+	rc_style->ythickness = 0;
+	gtk_widget_modify_style (window->details->location_button, 
+				 rc_style);
+
+	gtk_widget_show (window->details->location_button);
+
+	frame = gtk_frame_new (NULL);
+	gtk_widget_style_get (GTK_WIDGET (window->details->location_statusbar), 
+			      "shadow_type", &shadow_type, NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), shadow_type);
+	gtk_box_pack_start (GTK_BOX (window->details->location_statusbar), 
+			    frame, TRUE, TRUE, 0);
+	gtk_widget_show (frame);
+
+	gtk_container_add (GTK_CONTAINER (frame), 
+			   window->details->location_button);
+	
+	gtk_widget_set_sensitive (window->details->location_button, FALSE);
+	g_signal_connect (window->details->location_button, 
+			  "clicked", 
+			  G_CALLBACK (location_button_clicked_callback), window);
+	gtk_widget_show (window->details->location_statusbar);
 }
 
 static void
@@ -355,6 +577,7 @@ nautilus_spatial_window_class_init (NautilusSpatialWindowClass *class)
 	GTK_WIDGET_CLASS (class)->show = nautilus_spatial_window_show;
 	GTK_WIDGET_CLASS (class)->configure_event = nautilus_spatial_window_configure_event;
 	GTK_WIDGET_CLASS (class)->unrealize = nautilus_spatial_window_unrealize;
+	GTK_WIDGET_CLASS (class)->realize = nautilus_spatial_window_realize;
 
 	NAUTILUS_WINDOW_CLASS (class)->prompt_for_location = 
 		real_prompt_for_location;
@@ -367,4 +590,7 @@ nautilus_spatial_window_class_init (NautilusSpatialWindowClass *class)
 	NAUTILUS_WINDOW_CLASS (class)->close = 
 		real_window_close;
 	NAUTILUS_WINDOW_CLASS(class)->get_default_size = real_get_default_size;
+
+	NAUTILUS_WINDOW_CLASS(class)->set_throbber_active =
+		real_set_throbber_active;
 }
